/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.cell.core.client.form;

import java.text.ParseException;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import com.google.gwt.cell.client.Cell;
import com.google.gwt.cell.client.TextCell;
import com.google.gwt.cell.client.ValueUpdater;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.core.client.Scheduler.ScheduledCommand;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.dom.client.Style.Overflow;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.safehtml.shared.SafeHtmlUtils;
import com.google.gwt.text.shared.AbstractSafeHtmlRenderer;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.Event.NativePreviewEvent;
import com.google.gwt.user.client.Window;
import com.google.gwt.user.client.ui.RootPanel;
import com.sencha.gxt.core.client.GXT;
import com.sencha.gxt.core.client.Style.Anchor;
import com.sencha.gxt.core.client.Style.AnchorAlignment;
import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.resources.ThemeStyles;
import com.sencha.gxt.core.client.util.BaseEventPreview;
import com.sencha.gxt.core.client.util.DelayedTask;
import com.sencha.gxt.core.client.util.KeyNav;
import com.sencha.gxt.core.shared.event.GroupingHandlerRegistration;
import com.sencha.gxt.data.shared.LabelProvider;
import com.sencha.gxt.data.shared.ListStore;
import com.sencha.gxt.data.shared.Store;
import com.sencha.gxt.data.shared.Store.StoreFilter;
import com.sencha.gxt.data.shared.event.StoreDataChangeEvent;
import com.sencha.gxt.data.shared.event.StoreDataChangeEvent.StoreDataChangeHandler;
import com.sencha.gxt.data.shared.event.StoreUpdateEvent;
import com.sencha.gxt.data.shared.event.StoreUpdateEvent.StoreUpdateHandler;
import com.sencha.gxt.data.shared.loader.Loader;
import com.sencha.gxt.data.shared.loader.PagingLoadConfig;
import com.sencha.gxt.data.shared.loader.PagingLoadConfigBean;
import com.sencha.gxt.data.shared.loader.PagingLoadResult;
import com.sencha.gxt.data.shared.loader.PagingLoader;
import com.sencha.gxt.widget.core.client.ListView;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer;
import com.sencha.gxt.widget.core.client.container.VerticalLayoutContainer.VerticalLayoutData;
import com.sencha.gxt.widget.core.client.event.CollapseEvent;
import com.sencha.gxt.widget.core.client.event.ExpandEvent;
import com.sencha.gxt.widget.core.client.event.XEvent;
import com.sencha.gxt.widget.core.client.form.PropertyEditor;
import com.sencha.gxt.widget.core.client.selection.SelectionChangedEvent;
import com.sencha.gxt.widget.core.client.selection.SelectionChangedEvent.SelectionChangedHandler;
import com.sencha.gxt.widget.core.client.toolbar.PagingToolBar;

public class ComboBoxCell<T> extends TriggerFieldCell<T> {

  public enum QueryMode {
    REMOTE, LOCAL
  }

  /**
   * TriggerAction enum.
   */
  public enum TriggerAction {
    ALL, QUERY;
  }

  private class ComboPropertyEditor extends PropertyEditor<T> {

    @Override
    public T parse(CharSequence text) throws ParseException {
      return selectByValue(text == null ? "" : text.toString());
    }

    @Override
    public String render(T object) {
      return object == null ? "" : object.toString();
    }

  }
  private class Handler implements StoreDataChangeHandler<T>, StoreUpdateHandler<T> {

    @Override
    public void onDataChange(StoreDataChangeEvent<T> event) {
      if (lastContext != null) {
        onResultsLoad(lastContext, lastParent, lastValueUpdater, lastValue);
      }
    }

    @Override
    public void onUpdate(StoreUpdateEvent<T> event) {
      // TODO
    }
  }
  protected String lastQuery;
  protected ListStore<T> store;
  protected Loader<?, ?> loader;
  protected int pageSize;

  protected PagingToolBar pagingToolBar;
  protected LabelProvider<T> labelProvider, viewLabelProvider;
  private TriggerAction triggerAction = TriggerAction.QUERY;
  private boolean expanded;
  private String allQuery = "";
  private VerticalLayoutContainer listContainer;
  private ListView<T, ?> listView;
  private QueryMode mode = QueryMode.LOCAL;
  private int minChars = 0;
  private boolean useQueryCache = true;
  private T selectedItem;
  private int minListWidth = 70;
  private int maxHeight = 300;
  private boolean typeAhead;
  private int typeAheadDelay = 250;
  private StoreFilter<T> currentFilter;
  private BaseEventPreview eventPreview;
  private boolean forceSelection;
  private DelayedTask taTask, dqTask;
  private String lastSelectionText;
  private int queryDelay = 500;
  private KeyNav keyNav;
  private GroupingHandlerRegistration storeRegistration;

  private Handler storeHandler = new Handler();

  private Logger logger = Logger.getLogger(getClass().getName());

  @SuppressWarnings({"unchecked", "rawtypes"})
  public ComboBoxCell(ListStore<T> store, LabelProvider<T> labelProvider, final LabelProvider<T> viewLabelProvider) {
    this.labelProvider = labelProvider;
    this.viewLabelProvider = viewLabelProvider;

    listView = new ListView<T, String>(store, new ValueProvider<T, String>() {

      @Override
      public String getPath() {
        return "";
      }

      @Override
      public String getValue(T object) {
        return viewLabelProvider.getLabel(object);
      }

      @Override
      public void setValue(T object, String value) {
      }
    });

    listView.setCell((Cell) new TextCell(new AbstractSafeHtmlRenderer<String>() {
      @Override
      public SafeHtml render(String object) {
        return SafeHtmlUtils.fromTrustedString(object);
      }
    }));

    setPropertyEditor(new ComboPropertyEditor());

    initView(listView);
    init(store);

  }

  public ComboBoxCell(ListStore<T> store, LabelProvider<T> labelProvider, ListView<T, ?> view) {
    this.labelProvider = labelProvider;

    setPropertyEditor(new ComboPropertyEditor());

    initView(view);
    init(store);
  }
  
  public void clear() {
    getStore().removeFilters();
    boolean f = forceSelection;
    forceSelection = false;
    // TODO
    forceSelection = f;
  }

  public void clearSelections(Context context, XElement parent, ValueUpdater<T> updater) {
    setText(parent, "");
    lastSelectionText = "";
    applyEmptyText(context, parent);
    updater.update(null);
  }

  public void collapse(Context context, final XElement parent) {
    if (!expanded) {
      return;
    }

    eventPreview.remove();
    expanded = false;

    listView.getSelectionModel().deselectAll();

    RootPanel.get().remove(listContainer);
    fireEvent(context, new CollapseEvent());

    Scheduler.get().scheduleDeferred(new ScheduledCommand() {

      @Override
      public void execute() {
        getInputElement(parent).focus();
      }
    });

  }

  @SuppressWarnings({"rawtypes", "unchecked"})
  public void doQuery(Context context, XElement parent, ValueUpdater<T> updater, T value, String query, boolean force) {
    if (query == null) {
      query = "";
    }

    // fire before query

    if (force || query.length() >= minChars) {
      if (!useQueryCache || !query.equals(lastQuery)) {
        lastQuery = query;
        if (mode == QueryMode.LOCAL) {
          selectedItem = null;

          // get rid of other filters, in anticipation of adding the new one -
          // this will not cause the filters to be re-applied
          listView.getStore().setEnableFilters(false);
          if (currentFilter != null) {
            listView.getStore().getFilters().remove(currentFilter);
          }

          final String fq = query;

          // add a new filter
          currentFilter = new StoreFilter<T>() {
            public boolean select(Store<T> store, T parent, T item) {
              return itemMatchesQuery(item, fq);
            }
          };
          listView.getStore().addFilter(currentFilter);
          listView.getStore().setEnableFilters(true);

          onResultsLoad(context, parent, updater, value);
        } else {
          expand(context, parent, updater, value);
          PagingLoader<PagingLoadConfig, PagingLoadResult<?>> l = (PagingLoader) loader;
          l.load(getParams(query));
        }
      } else {
        selectedItem = null;
        onResultsLoad(context, parent, updater, value);
      }
    }
  }

  public void expand(Context context, final XElement parent, ValueUpdater<T> updater, T value) {
    if (expanded) {
      return;
    }

    saveContext(context, parent, null, updater, value);

    RootPanel.get().add(listContainer);
    listContainer.getElement().updateZIndex(0);

    eventPreview.add();
    expanded = true;
    restrict(parent);

    Scheduler.get().scheduleDeferred(new ScheduledCommand() {

      @Override
      public void execute() {
        restrict(parent);
      }
    });

    fireEvent(context, new ExpandEvent());
  }

  public LabelProvider<T> getLabelProvider() {
    return labelProvider;
  }

  public ListView<T, ?> getListView() {
    return listView;
  }

  public Loader<?, ?> getLoader() {
    return loader;
  }

  /**
   * Returns the min characters used for autocompete and typeahead.
   * 
   * @return the minimum number of characters
   */
  public int getMinChars() {
    return minChars;
  }

  public QueryMode getMode() {
    return mode;
  }

  /**
   * Returns the page size.
   * 
   * @return the page size
   */
  public int getPageSize() {
    return pageSize;
  }

  /**
   * Returns the combo's paging tool bar.
   * 
   * @return the tool bar
   */
  public PagingToolBar getPagingToolBar() {
    return pagingToolBar;
  }

  public ListStore<T> getStore() {
    return store;
  }

  public TriggerAction getTriggerAction() {
    return triggerAction;
  }

  public int getTypeAheadDelay() {
    return typeAheadDelay;
  }

  public boolean isExpanded() {
    return expanded;
  }

  public boolean isForceSelection() {
    return forceSelection;
  }

  public boolean isTypeAhead() {
    return typeAhead;
  }

  public void onBrowserEvent(Cell.Context context, Element parent, T value, NativeEvent event,
      ValueUpdater<T> valueUpdater) {
    super.onBrowserEvent(context, parent, value, event, valueUpdater);
    keyNav.handleEvent(event);
  }

  @Override
  public void render(Context context, T value, SafeHtmlBuilder sb) {
    String v = "";

    if (value != null) {
      v = labelProvider != null ? labelProvider.getLabel(value) : getPropertyEditor().render(value);
    }

    FieldViewData viewData = checkViewData(context, v);
    String s = (viewData != null) ? viewData.getCurrentValue() : v;

    FieldAppearanceOptions options = new FieldAppearanceOptions(width, height, isReadOnly());
    options.setName(name);
    options.setEmptyText(getEmptyText());
    options.setHideTrigger(isHideTrigger());
    options.setEditable(isEditable());

    appearance.render(sb, s == null ? "" : s, options);
  }

  /**
   * Select an item in the dropdown list by its numeric index in the list. This
   * function does NOT cause the select event to fire. The list must expanded
   * for this function to work, otherwise use #setValue.
   * 
   * @param index the index of the item to select
   */
  public void select(int index) {
    if (index != -1 && index < getListView().getElements().size()) {
      T item = getStore().get(index);
      selectedItem = item;
      getListView().getSelectionModel().select(item, false);
      getListView().getElement(index).scrollIntoView();
    }
  }

  /**
   * Select an item in the dropdown list. This function does NOT cause the
   * select event to fire. The list must expanded for this function to work,
   * otherwise use #setValue.
   * 
   * @param item the item to select
   */
  public void select(T item) {
    if (item != null) {
      selectedItem = item;
      int index = getStore().indexOf(item);
      if (index != -1) {
        select(index);
      }
    }
  }

  public void setForceSelection(boolean forceSelection) {
    this.forceSelection = forceSelection;
  }

  public void setLabelProvider(LabelProvider<T> labelProvider) {
    this.labelProvider = labelProvider;
  }

  public void setLoader(Loader<?, ?> loader) {
    setMode(loader == null ? QueryMode.LOCAL : QueryMode.REMOTE);
    this.loader = loader;
  }

  /**
   * Sets the minimum number of characters the user must type before
   * autocomplete and typeahead active (defaults to 4 if remote, or 0 if local).
   * 
   * @param minChars
   */
  public void setMinChars(int minChars) {
    this.minChars = minChars;
  }

  public void setMode(QueryMode mode) {
    this.mode = mode;
    if (mode == QueryMode.REMOTE) {
      minChars = 4;
    }
  }

  /**
   * Sets the page size. Only applies when using a paging toolbar.
   * 
   * @param pageSize the page size
   */
  @SuppressWarnings({"unchecked"})
  public void setPageSize(int pageSize) {
    this.pageSize = pageSize;
    if (pageSize > 0) {
      if (pagingToolBar != null) {
        pagingToolBar.setPageSize(pageSize);
      } else {
        pagingToolBar = createPagingToolBar(pageSize);
        pagingToolBar.addStyleName(ThemeStyles.getStyle().borderTop());
        pagingToolBar.bind((PagingLoader<PagingLoadConfig, ?>) loader);
        listContainer.add(pagingToolBar, new VerticalLayoutData(1, -1));
        listContainer.setShadow(false);
      }
    } else {
      pagingToolBar = null;
    }
  }

  /**
   * Sets the combo's store.
   * 
   * @param store the store
   */
  public void setStore(ListStore<T> store) {
    bindStore(store);
  }

  public void setTriggerAction(TriggerAction triggerAction) {
    this.triggerAction = triggerAction;
  }

  public void setTypeAhead(boolean typeAhead) {
    this.typeAhead = typeAhead;
    if (typeAhead && taTask == null) {
      taTask = new DelayedTask() {
        @Override
        public void onExecute() {
          onTypeAhead(lastParent);
        }
      };
    } else if (!typeAhead && taTask != null) {
      taTask.cancel();
      taTask = null;
    }
  }

  public void setTypeAheadDelay(int typeAheadDelay) {
    this.typeAheadDelay = typeAheadDelay;
  }

  protected void bindStore(ListStore<T> store) {
    if (this.store != null) {
      storeRegistration.removeHandler();
      this.store = null;
      if (listView != null) {
        listView.setStore(null);
      }
    }
    if (store != null) {
      this.store = store;
      if (listView != null) {
        listView.setStore(store);
      }
      if (storeRegistration == null) {
        storeRegistration = new GroupingHandlerRegistration();
      }
      storeRegistration.add(store.addStoreDataChangeHandler(storeHandler));
      storeRegistration.add(store.addStoreUpdateHandler(storeHandler));
    }
  }

  protected void collapseIf(NativePreviewEvent pe) {
    Element target = pe.getNativeEvent().getEventTarget().cast();
    if (!listContainer.getElement().isOrHasChild(target) && !lastParent.isOrHasChild(target)) {
      collapse(lastContext, lastParent);
    }
    // if (!list.el().isOrHasChild(pe.getTarget()) &&
    // !el().isOrHasChild(pe.getTarget())) {
    // collapse();
    // }
  }

  protected PagingToolBar createPagingToolBar(int pageSize) {
    return new PagingToolBar(pageSize);
  }

  protected void doForce(Context context, XElement parent, T value) {
    if (forceSelection) {
      boolean f = forceSelection;
      forceSelection = false;
      String rv = getText(parent);
      if (isAllowBlank() && (rv == null || rv.equals(""))) {
        forceSelection = f;
        return;
      }

      T v = selectByValue(rv);

      if (v == null) {
        if (lastSelectionText != null && !"".equals(lastSelectionText)) {
          setText(parent, lastSelectionText);
          if (getMode() == QueryMode.LOCAL) {
            // we need to filter the store here, so that the store only contains
            // the items needed

            // we adjust the lastQuery, so doLoad does not use an old value
            lastQuery = lastSelectionText;
            // store.filter(getDisplayField(), getRawValue());
          }

        } else {
          setText(parent, "");
          applyEmptyText(context, parent);
        }
      }
      forceSelection = f;
    }
  }

  protected PagingLoadConfig getParams(String query) {
    PagingLoadConfig config = null;
    if (loader.isReuseLoadConfig()) {
      config = (PagingLoadConfig) loader.getLastLoadConfig();
    } else {
      config = new PagingLoadConfigBean();
    }
    config.setLimit(pageSize);
    config.setOffset(0);
    // config.set("query", query);
    return config;
  }

  protected void init(ListStore<T> store) {
    listContainer = new VerticalLayoutContainer();
    listContainer.getElement().makePositionable(true);
    listContainer.setBorders(true);
    listContainer.setShadow(true);
    listContainer.add(listView, new VerticalLayoutData(1, -1));

    bindStore(store);

    eventPreview = new BaseEventPreview() {
      protected boolean onPreview(NativePreviewEvent pe) {
        Element target = pe.getNativeEvent().getEventTarget().cast();

        switch (pe.getTypeInt()) {
          case Event.ONSCROLL:
          case Event.ONMOUSEWHEEL:
          case Event.ONMOUSEUP:
            collapseIf(pe);
            break;
          case Event.ONMOUSEDOWN:
            if (listView.getElement().isOrHasChild(target)) {
              if (pagingToolBar != null && !pagingToolBar.getElement().isOrHasChild(target)) {
                onViewClick(lastParent, pe.getNativeEvent(), true, false);
              } else {
                onViewClick(lastParent, pe.getNativeEvent(), true, false);
              }
            }
            break;
        }

        NativeEvent e = pe.getNativeEvent();

        if (pe.getTypeInt() == KeyNav.getKeyEvent() && expanded) {
          if (e.getKeyCode() == KeyCodes.KEY_ENTER) {
            if (pagingToolBar != null && pagingToolBar.getElement().isOrHasChild(target)) {
              return true;
            }
            e.stopPropagation();
            e.preventDefault();
            onViewClick(lastParent, e, false, true);
          }
        }
        return true;
      }
    };
    eventPreview.setAutoHide(false);

    dqTask = new DelayedTask() {

      @Override
      public void onExecute() {
        doQuery(lastContext, lastParent, lastValueUpdater, lastValue, getText(lastParent), false);
      }
    };

    keyNav = new KeyNav() {
      @Override
      public void onDown(NativeEvent evt) {
        if (!isReadOnly()) {
          evt.stopPropagation();
          if (!isExpanded()) {
            onTriggerClick(lastContext, lastParent, evt, lastValue, lastValueUpdater);
          } else {
            selectNext();
          }
        }
      }

      public void onEsc(NativeEvent evt) {
        if (isExpanded()) {
          evt.stopPropagation();
          collapse(lastContext, lastParent);
        }
      };

      public void onTab(NativeEvent evt) {
        if (isExpanded()) {
          logger.log(Level.FINEST, "onTab");
          onViewClick(lastParent, evt, false, true);
        }
      };

      public void onUp(NativeEvent evt) {
        if (isExpanded()) {
          evt.stopPropagation();
          selectPrev();
        }
      };
    };
  }

  protected void initView(ListView<T, ?> listView) {
    this.listView = listView;

    listView.setBorders(false);
    listView.setSelectOnOver(true);
    listView.getSelectionModel().addSelectionChangedHandler(new SelectionChangedHandler<T>() {
      @Override
      public void onSelectionChanged(SelectionChangedEvent<T> event) {
        List<T> sel = event.getSelection();
        if (sel.size() > 0) {
          selectedItem = sel.get(0);
        } else {
          selectedItem = null;
        }

      }
    });

    listView.getElement().getStyle().setOverflowX(Overflow.HIDDEN);
  }

  @Override
  protected boolean isFocusClick(XElement parent, XElement target) {
    boolean result = parent.isOrHasChild(target) || listView.getElement().isOrHasChild(target);
    if (!result && pagingToolBar != null) {
      if (pagingToolBar.getElement().isOrHasChild(target)) {
        return true;
      }
    }
    return result;
  }

  protected boolean itemMatchesQuery(T item, String query) {
    String value = getRenderedValue(item);
    if (value != null) {
      return value.toLowerCase().startsWith(query.toLowerCase());
    }
    return false;
  }

  protected void onEmptyResults(Context context, XElement parent) {
    collapse(context, parent);
  }

  @Override
  protected void onKeyUp(Context context, Element parent, T value, NativeEvent event, ValueUpdater<T> valueUpdater) {
    super.onKeyUp(context, parent, value, event, valueUpdater);

    int kc = event.getKeyCode();

    if (!isReadOnly() && isEditable()
        && (!event.<XEvent> cast().isSpecialKey() || kc == KeyCodes.KEY_BACKSPACE || kc == 46)) {
      dqTask.delay(queryDelay);
    }
  };

  protected void onResultsLoad(Context context, XElement parent, ValueUpdater<T> updater, T value) {
    if (!hasFocus(context, parent)) {
      return;
    }

    if (listView.getStore().size() == 0) {
      onEmptyResults(context, parent);
    } else {

      if (expanded) {
        restrict(parent);
      } else {
        expand(context, parent, updater, value);
      }

      if (lastQuery != null && lastQuery.equals(allQuery)) {
        if (isEditable()) {
          selectAll(parent);
        }
      } else {
        if (typeAhead) {
          taTask.delay(typeAheadDelay);
        }
      }

      // select an element in the listview based on the current text
      if (selectByValue(getText(parent)) == null) {
        select(0);
      }
    }
  }

  protected void onSelect(T item) {
    FieldViewData viewData = ensureViewData(lastContext, lastParent);

    String rv = getRenderedValue(item);

    this.lastSelectionText = rv;

    if (viewData != null) {
      viewData.setCurrentValue(getRenderedValue(item));
    }
    getInputElement(lastParent).setValue(getRenderedValue(item));

    // collapsing non deferred causes trigger field mouse down preview
    // to think a focus click has occurred which causing the field to be
    // blurred after value changed
    if (GXT.isIE()) {
      Scheduler.get().scheduleDeferred(new ScheduledCommand() {

        @Override
        public void execute() {
          collapse(lastContext, lastParent.<XElement> cast());
        }
      });

    } else {
      collapse(lastContext, lastParent.<XElement> cast());
    }

  }

  @Override
  protected void onTriggerClick(Context context, XElement parent, NativeEvent event, T value, ValueUpdater<T> updater) {
    super.onTriggerClick(context, parent, event, value, updater);
    if (expanded) {
      collapse(context, parent);
    } else {
      onFocus(context, parent, value, event, updater);
      if (triggerAction == TriggerAction.ALL) {
        doQuery(context, parent, updater, value, allQuery, true);
      } else {
        doQuery(context, parent, updater, value, getText(parent.<XElement> cast()), true);
      }
    }
    getInputElement(parent).focus();
  }

  protected void onTypeAhead(XElement parent) {
    if (store.size() > 0) {
      T m = store.get(0);
      String newValue = getRenderedValue(m);
      int len = newValue.length();
      int selStart = getText(lastParent).length();
      if (selStart != len) {
        setText(parent, newValue);
        select(parent, selStart, newValue.length());
      }
    }
  }

  protected void onViewClick(final XElement parent, NativeEvent event, boolean focus, boolean takeSelected) {
    int idx = -1;

    Element elem = listView.findElement((Element) event.getEventTarget().cast());
    if (elem != null) {
      idx = listView.indexOf(elem);
    } else if (elem == null && !takeSelected) {
      return;

    } else {
      T sel = listView.getSelectionModel().getSelectedItem();
      if (sel != null) {
        idx = store.indexOf(sel);
      }
    }
    if (idx != -1) {
      T sel = store.get(idx);
      onSelect(sel);
    }

    if (focus) {
      Scheduler.get().scheduleDeferred(new ScheduledCommand() {

        @Override
        public void execute() {
          logger.log(Level.FINEST, "onViewClick parent.focus()");
          parent.focus();
        }
      });
    }

  }

  protected void restrict(XElement parent) {
    XElement wrapper = parent.getFirstChildElement().cast();
    int width = Math.max(wrapper.getWidth(true), minListWidth);

    listContainer.getElement().setVisibility(false);
    listView.setHeight("auto");
    listContainer.setHeight("auto");

    int fh = 0;
    int h = fh + listContainer.getOffsetHeight();
    int mH = Math.min(maxHeight, Window.getClientHeight() - 10);
    h = Math.min(h, mH);

    listContainer.getElement().makePositionable(true);
    listContainer.setPixelSize(width, h);
    listContainer.getElement().alignTo(wrapper, new AnchorAlignment(Anchor.TOP_LEFT, Anchor.BOTTOM_LEFT, true), null);
    listContainer.getElement().setVisibility(true);

    listView.setHeight(h - 2 - (pagingToolBar != null ? pagingToolBar.getOffsetHeight() : 0));

    Scheduler.get().scheduleDeferred(new ScheduledCommand() {
      @Override
      public void execute() {
        listContainer.sync(true);
      }
    });

  }

  protected T selectByValue(String value) {
    int count = store.size();
    for (int i = 0; i < count; i++) {
      T item = store.get(i);
      String v = getRenderedValue(item);
      if (v != null && v.equals(value)) {
        select(item);
        return item;
      }
    }
    return null;
  }

  protected void selectNext() {
    int count = getStore().size();
    if (count > 0) {
      int selectedIndex = getStore().indexOf(selectedItem);
      if (selectedIndex == -1) {
        select(0);
      } else if (selectedIndex < count - 1) {
        select(selectedIndex + 1);
      }
    }
  }

  protected void selectPrev() {
    int count = store.size();
    if (count > 0) {
      int selectedIndex = store.indexOf(selectedItem);
      if (selectedIndex == -1) {
        select(0);
      } else if (selectedIndex != 0) {
        select(selectedIndex - 1);
      }
    }
  }

  @Override
  protected void triggerBlur(Context context, XElement parent, T value, ValueUpdater<T> valueUpdater) {
    doForce(context, parent, value);
    dqTask.cancel();
    collapse(context, parent);

    super.triggerBlur(context, parent, value, valueUpdater);
  }

  private String getRenderedValue(T item) {
    return labelProvider != null ? labelProvider.getLabel(item) : getPropertyEditor().render(item);
  }

}
