/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.cell.core.client.form;

import com.google.gwt.cell.client.Cell;
import com.google.gwt.cell.client.ValueUpdater;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.NativeEvent;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.widget.core.client.form.NumberPropertyEditor;

public class SpinnerFieldCell<N extends Number> extends NumberInputCell<N> {

  public interface SpinnerFieldAppearance extends TwinTriggerFieldAppearance {

  }
  private Number minValue = Double.MIN_VALUE;
  private Number maxValue = Double.MAX_VALUE;

  public SpinnerFieldCell(NumberPropertyEditor<N> propertyEditor) {
    this(propertyEditor, GWT.<SpinnerFieldAppearance> create(SpinnerFieldAppearance.class));
  }

  public SpinnerFieldCell(NumberPropertyEditor<N> propertyEditor, SpinnerFieldAppearance appearance) {
    super(propertyEditor, appearance);
  }

  /**
   * Returns the fields max value.
   * 
   * @param context the context
   * @return the max value
   */
  public Number getMaxValue(Context context) {
    return maxValue;
  }

  /**
   * Returns the field's minimum value.
   * 
   * @param context the context
   * @return the min value
   */
  public Number getMinValue(Context context) {
    return minValue;
  }

  /**
   * Sets the field's max allowable value.
   * 
   * @param maxValue the max value
   */
  public void setMaxValue(Number maxValue) {
    this.maxValue = maxValue.doubleValue();
    // if (rendered && maxValue.doubleValue() != Double.MAX_VALUE) {
    // getInputEl().dom.setAttribute("aria-valuemax", "" + maxValue);
    // }
  }

  /**
   * Sets the increment that should be used (defaults to 1d).
   * 
   * @param increment the increment to set.
   */
  public void setIncrement(N increment) {
    getPropertyEditor().setIncrement(increment);
  }

  /**
   * Sets the increment value.
   * 
   * @param context the context
   * @return the increment
   */
  public N getIncrement(Context context) {
    return getPropertyEditor().getIncrement();
  }

  /**
   * Sets the field's minimum allowed value.
   * 
   * @param minValue the minimum value
   */
  public void setMinValue(Number minValue) {
    this.minValue = minValue.doubleValue();
    // if (rendered && maxValue.doubleValue() != Double.NEGATIVE_INFINITY) {
    // getInputEl().dom.setAttribute("aria-valuemin", "" + minValue);
    // }
  }

  protected void doSpin(Cell.Context context, XElement parent, N value, ValueUpdater<N> updater, boolean up) {
    if (!isReadOnly()) {
      if (up) {
        N newVal = getPropertyEditor().incr(value);
        updater.update(newVal);
        setValue(context, parent, newVal);
        // setValue(context, parent,
        // Math.max(minValue.doubleValue(), Math.min(d +
        // increment.doubleValue(), maxValue.doubleValue())));
      } else {
        N newVal = getPropertyEditor().decr(value);
        updater.update(newVal);
        setValue(context, parent, newVal);
        // setValue(
        // context,
        // parent,
        // Math.max(minValue.doubleValue(), Math.min(
        // allowNegative ? d - increment.doubleValue() : Math.max(0, d -
        // increment.doubleValue()),
        // maxValue.doubleValue())));
      }
    }

  }

  @Override
  protected void onTriggerClick(Cell.Context context, XElement parent, NativeEvent event, N value,
      ValueUpdater<N> updater) {
    super.onTriggerClick(context, parent, event, value, updater);
    doSpin(context, parent, value, updater, true);
  }

  @Override
  protected void onTwinTriggerClick(Cell.Context context, XElement parent, NativeEvent event, N value,
      ValueUpdater<N> updater) {
    super.onTwinTriggerClick(context, parent, event, value, updater);
    doSpin(context, parent, value, updater, false);
  }

}
