/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.core.rebind;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import com.google.gwt.core.ext.GeneratorContext;
import com.google.gwt.core.ext.TreeLogger;
import com.google.gwt.core.ext.TreeLogger.Type;
import com.google.gwt.core.ext.UnableToCompleteException;
import com.google.gwt.core.ext.typeinfo.JClassType;
import com.google.gwt.dev.util.Name;
import com.sencha.gxt.core.client.XTemplates;
import com.sencha.gxt.core.client.XTemplates.FormatterFactories;
import com.sencha.gxt.core.client.XTemplates.FormatterFactory;
import com.sencha.gxt.core.client.XTemplates.FormatterFactoryMethod;

public class FormatCollecter {
  private final TreeLogger logger;
  @SuppressWarnings("unused")
  private final GeneratorContext context;
  private final Map<String, String> formatters = new HashMap<String, String>();
  private final Map<String, String> defaultArgs = new HashMap<String, String>();

  public FormatCollecter(GeneratorContext context, TreeLogger l, JClassType toGenerate)
      throws UnableToCompleteException {
    this.logger = l.branch(Type.DEBUG, "Collecting formatters for " + toGenerate.getName());
    this.context = context;

    // Collect factories first
    List<FormatterFactory> foundFactories = new ArrayList<XTemplates.FormatterFactory>();
    for (JClassType type : toGenerate.getFlattenedSupertypeHierarchy()) {
      if (type.isAnnotationPresent(FormatterFactories.class)) {
        foundFactories.addAll(Arrays.asList(type.getAnnotation(FormatterFactories.class).value()));
      }
    }
    for (FormatterFactory f : foundFactories) {
      if (!f.name().equals("")) {
        registerFactory(f.name(), f.factory(), "getFormat", "");
      } else if (f.methods().length != 0) {
        for (FormatterFactoryMethod method : f.methods()) {
          registerFactory(method.name(), f.factory(), method.method(), method.defaultArgs());
        }
      } else {
        logger.log(Type.ERROR, "Formatter factory defined, but no way to access it (no name, no methods declared): "
            + f.factory());
        throw new UnableToCompleteException();
      }
    }
  }

  private void registerFactory(String name, Class<?> factory, String method, String defaultArgs) {
    // Don't overwrite an existing formatter
    if (formatters.containsKey(name)) {
      logger.log(Type.WARN,
          "Template already has a formatter with name " + name + ". Not registering " + factory.getName() + ".method");
    } else {
      // if method is static
      formatters.put(name, String.format("%1$s.%2$s(%3$s)", Name.getSourceNameForClass(factory), method, "%1$s"));
      // TODO support non-static factory methods?

      this.defaultArgs.put(name, defaultArgs);
    }
  }

  public String getFormatter(String name, String params) throws UnableToCompleteException {
    if (params == null) {
      // check for defaults
      params = defaultArgs.get(name);
    }
    String format = formatters.get(name);
    if (format == null) {
      logger.log(Type.ERROR, "Formatter with name " + name + " not found");
      throw new UnableToCompleteException();
    }
    return String.format(format, params);
  }

}
