/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.dnd.core.client;

import java.util.List;

import com.google.gwt.dom.client.Element;
import com.sencha.gxt.core.client.util.Format;
import com.sencha.gxt.dnd.core.client.DND.Operation;
import com.sencha.gxt.messages.client.DefaultMessages;
import com.sencha.gxt.widget.core.client.ListView;

public class ListViewDragSource<T> extends DragSource {

  public interface ListViewDragSourceMessages {

    String itemsSelected(int items);
  }

  protected class DefaultListViewDragSourceMessages implements ListViewDragSourceMessages {

    public String itemsSelected(int items) {
      return DefaultMessages.getMessages().listField_itemsSelected(items);
    }

  }

  protected ListView<T, ?> listView;

  protected ListViewDragSourceMessages messages;

  public ListViewDragSource(ListView<T, ?> listView) {
    super(listView);

    this.listView = listView;
  }

  /**
   * Returns the sources list view.
   * 
   * @return the list view
   */
  public ListView<T, ?> getListView() {
    return listView;
  }

  public ListViewDragSourceMessages getMessages() {
    if (messages == null) {
      messages = new DefaultListViewDragSourceMessages();
    }
    return messages;
  }

  public void setMessages(ListViewDragSourceMessages messages) {
    this.messages = messages;
  }

  @SuppressWarnings("unchecked")
  @Override
  protected void onDragDrop(DndDropEvent event) {
    if (event.getOperation() == Operation.MOVE) {
      Object data = event.getData();
      if (data instanceof List) {
        List<T> list = (List<T>) data;
        for (T item : list) {
          listView.getStore().remove(item);
        }
      }
    }
  }

  @Override
  protected void onDragStart(DndDragStartEvent event) {
    Element r = listView.findElement(event.getDragStartEvent().getStartElement());
    if (r == null) {
      event.setCancelled(true);
      return;
    }
    List<T> sel = listView.getSelectionModel().getSelectedItems();
    if (sel.size() > 0) {
      event.setCancelled(false);
      event.setData(sel);

      if (getStatusText() == null) {
        event.getStatusProxy().update(getMessages().itemsSelected(sel.size()));
      } else {
        event.getStatusProxy().update(Format.substitute(getStatusText(), sel.size()));
      }
    }
  }

}
