/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.theme.base.client.field;

import com.google.gwt.core.client.GWT;
import com.google.gwt.resources.client.ClientBundle;
import com.google.gwt.resources.client.CssResource;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.sencha.gxt.core.client.XTemplates;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.widget.core.client.form.FieldLabel;
import com.sencha.gxt.widget.core.client.form.FieldLabel.FieldLabelAppearance;
import com.sencha.gxt.widget.core.client.form.FormPanel.LabelAlign;

public class FieldLabelDefaultAppearance implements FieldLabelAppearance {

  public interface Resources extends ClientBundle {
    @Source("FieldLabel.css")
    Style css();
  }
  public interface Style extends CssResource {

    String fieldLabel();

    String fieldElement();

    String fieldItem();

    String hideLabel();

    String fieldItemLabelTop();
  }

  interface FieldLabelTemplate extends XTemplates {
    @XTemplate(source = "FieldLabel.html")
    SafeHtml render(String id, Style style, boolean isTop, int labelWidth, int padding);
  }

  private final Resources resources;
  private FieldLabelTemplate template;
  private Style style;
  @SuppressWarnings("unused")
  private int labelAdjust;

  public FieldLabelDefaultAppearance() {
    this(GWT.<Resources> create(Resources.class), GWT.<FieldLabelTemplate> create(FieldLabelTemplate.class));
  }

  public FieldLabelDefaultAppearance(Resources resources, FieldLabelTemplate template) {
    this.resources = resources;
    this.template = template;

    this.style = resources.css();

    style.ensureInjected();
  }

  public XElement getLabelElement(XElement parent) {
    return parent.selectNode("label");
  }

  @Override
  public XElement getChildElementWrapper(XElement parent) {
    return parent.selectNode("div." + resources.css().fieldElement());
  }

  @Override
  public void render(SafeHtmlBuilder sb, String id, FieldLabel label) {
    int labelWidth = label.getLabelWidth();
    LabelAlign align = label.getLabelAlign();

    int pad = label.getLabelPad() != 0 ? label.getLabelPad() : 5;
    labelAdjust = align == LabelAlign.TOP ? 0 : label.getLabelWidth() + pad;

    sb.append(template.render(id, style, align == LabelAlign.TOP, labelWidth, pad));
  }

  @Override
  public void onLabelAlign(XElement parent, FieldLabel label) {
    switch (label.getLabelAlign()) {
      case TOP:
        parent.addClassName(style.fieldItemLabelTop());
        break;
    }
  }
}