/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client;

import com.sencha.gxt.cell.core.client.SliderCell;
import com.sencha.gxt.theme.blue.client.slider.BlueSliderHorizontalAppearance;
import com.sencha.gxt.theme.blue.client.slider.BlueSliderVerticalAppearance;
import com.sencha.gxt.widget.core.client.form.Field;

public class Slider extends Field<Integer> {

  protected final SliderCell cell;

  public Slider() {
    super(new SliderCell());
    cell = (SliderCell)getCell();
    setAllowTextSelection(false);
    redraw();
  }

  public Slider(boolean vertical) {
    super(new SliderCell(vertical ? new BlueSliderVerticalAppearance() : new BlueSliderHorizontalAppearance()));
    cell = (SliderCell) getCell();
    redraw();
  }

  public Slider(SliderCell cell) {
    super(cell);
    this.cell = cell;
    redraw();
  }
  
  /**
   * Returns the increment.
   * 
   * @return the increment
   */
  public int getIncrement() {
    return cell.getIncrement();
  }
  
  /**
   * Returns the max value (defaults to 100).
   * 
   * @return the max value
   */
  public int getMaxValue() {
    return cell.getMaxValue();
  }

  /**
   * Returns the tool tip message.
   * 
   * @return the tool tip message
   */
  public String getMessage() {
    return cell.getMessage();
  }
  
  /**
   * Returns the minimum value (defaults to 0).
   * 
   * @return the minimum value
   */
  public int getMinValue() {
    return cell.getMinValue();
  }
  
  /**
   * How many units to change the slider when adjusting by drag and drop. Use
   * this option to enable 'snapping' (default to 10).
   * 
   * @param increment the increment
   */
  public void setIncrement(int increment) {
    cell.setIncrement(increment);
  }
  
  /**
   * Sets the max value (defaults to 100).
   * 
   * @param maxValue the max value
   */
  public void setMaxValue(int maxValue) {
    cell.setMaxValue(maxValue);
  }
  
  /**
   * Sets the tool tip message (defaults to '{0}'). "{0} will be substituted
   * with the current slider value.
   * 
   * @param message the tool tip message
   */
  public void setMessage(String message) {
    cell.setMessage(message);
  }
  
  /**
   * Sets the minimum value (defaults to 0).
   * 
   * @param minValue the minimum value
   */
  public void setMinValue(int minValue) {
    cell.setMinValue(minValue);
  }

}
