/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.form;

import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.sencha.gxt.core.client.Style.Side;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.util.Size;
import com.sencha.gxt.widget.core.client.Collapsible;
import com.sencha.gxt.widget.core.client.ComponentHelper;
import com.sencha.gxt.widget.core.client.button.ToolButton;
import com.sencha.gxt.widget.core.client.container.SimpleContainer;
import com.sencha.gxt.widget.core.client.event.BeforeCollapseEvent;
import com.sencha.gxt.widget.core.client.event.BeforeCollapseEvent.BeforeCollapseHandler;
import com.sencha.gxt.widget.core.client.event.BeforeCollapseEvent.HasBeforeCollapseHandlers;
import com.sencha.gxt.widget.core.client.event.BeforeExpandEvent;
import com.sencha.gxt.widget.core.client.event.BeforeExpandEvent.BeforeExpandHandler;
import com.sencha.gxt.widget.core.client.event.BeforeExpandEvent.HasBeforeExpandHandlers;
import com.sencha.gxt.widget.core.client.event.CollapseEvent;
import com.sencha.gxt.widget.core.client.event.CollapseEvent.CollapseHandler;
import com.sencha.gxt.widget.core.client.event.CollapseEvent.HasCollapseHandlers;
import com.sencha.gxt.widget.core.client.event.ExpandEvent;
import com.sencha.gxt.widget.core.client.event.ExpandEvent.ExpandHandler;
import com.sencha.gxt.widget.core.client.event.ExpandEvent.HasExpandHandlers;

public class FieldSet extends SimpleContainer implements HasBeforeExpandHandlers,
    HasExpandHandlers, HasBeforeCollapseHandlers, HasCollapseHandlers,
    Collapsible {

  public interface FieldSetAppearance {
    XElement getBodyElement(XElement parent);

    XElement getTextElement(XElement parent);

    XElement getToolElement(XElement parent);

    void onCollapse(XElement parent, boolean collapse);

    void render(SafeHtmlBuilder sb);
  }

  private final FieldSetAppearance appearance;
  private boolean collapsed, collapsible;
  private ToolButton collapseBtn;

  public FieldSet() {
    this(GWT.<FieldSetAppearance> create(FieldSetAppearance.class));
  }

  public FieldSet(FieldSetAppearance appearance) {
    super(true);

    this.appearance = appearance;

    SafeHtmlBuilder builder = new SafeHtmlBuilder();
    this.appearance.render(builder);

    setElement(XDOM.create(builder.toSafeHtml()));
  }

  @Override
  public HandlerRegistration addBeforeCollapseHandler(BeforeCollapseHandler handler) {
    return addHandler(handler, BeforeCollapseEvent.getType());
  }

  @Override
  public HandlerRegistration addBeforeExpandHandler(BeforeExpandHandler handler) {
    return addHandler(handler, BeforeExpandEvent.getType());
  }

  @Override
  public HandlerRegistration addCollapseHandler(CollapseHandler handler) {
    return addHandler(handler, CollapseEvent.getType());
  }

  @Override
  public HandlerRegistration addExpandHandler(ExpandHandler handler) {
    return addHandler(handler, ExpandEvent.getType());
  }

  @Override
  public void collapse() {
    if (collapsible) {
      if (fireCancellableEvent(new BeforeCollapseEvent())) {
        this.collapsed = true;
        appearance.getBodyElement(getElement()).setVisible(false);
        appearance.onCollapse(getElement(), true);
        collapseBtn.changeStyle(ToolButton.DOWN);
        fireEvent(new CollapseEvent());
      }
    }
  }

  public void expand() {
    if (collapsible) {
      if (fireCancellableEvent(new BeforeCollapseEvent())) {
        this.collapsed = false;
        appearance.getBodyElement(getElement()).setVisible(true);
        appearance.onCollapse(getElement(), false);
        collapseBtn.changeStyle(ToolButton.UP);
        fireEvent(new ExpandEvent());
      }
    }
  }

  public String getHeadingHtml() {
    return appearance.getTextElement(getElement()).getInnerHTML();
  }

  public String getHeadingText() {
    return appearance.getTextElement(getElement()).getInnerText();
  }

  public boolean isCollapsed() {
    return collapsed;
  }

  public boolean isCollapsible() {
    return collapsible;
  }

  public void setCollapsed(boolean collapsed) {
    this.collapsed = collapsed;
  }

  public void setCollapsible(boolean collapsible) {
    this.collapsible = collapsible;
  }
  
  /**
   * Sets the panel's expand state.
   * 
   * @param expand true to expand
   */
  public void setExpanded(boolean expand) {
    if (expand) {
      expand();
    } else {
      collapse();
    }
  }
  
  public void setHeadingHtml(String heading) {
    appearance.getTextElement(getElement()).setInnerHTML(heading);
  }
  
  public void setHeadingText(String heading) {
    appearance.getTextElement(getElement()).setInnerText(heading);
  }

  @Override
  protected void doAttachChildren() {
    super.doAttachChildren();
    ComponentHelper.doAttach(collapseBtn);
  }

  @Override
  protected void doDetachChildren() {
    super.doDetachChildren();
    ComponentHelper.doDetach(collapseBtn);
  }

  @Override
  protected XElement getContainerTarget() {
    return appearance.getBodyElement(getElement());
  }

  @Override
  protected void notifyHide() {
    if (!collapsed) {
      super.notifyHide();
    }
  }

  @Override
  protected void notifyShow() {
    if (!collapsed) {
      super.notifyShow();
    }
  }

  @Override
  protected void onAfterFirstAttach() {
    super.onAfterFirstAttach();
    if (collapsible && collapseBtn == null) {
      collapseBtn = new ToolButton(ToolButton.UP);
      collapseBtn.addClickHandler(new ClickHandler() {

        @Override
        public void onClick(ClickEvent event) {
          setExpanded(isCollapsed());
        }
      });
      ComponentHelper.doAttach(collapseBtn);
      appearance.getToolElement(getElement()).appendChild(collapseBtn.getElement());
    }
  }
  
  @Override
  protected void onResize(int width, int height) {
    super.onResize(width, height);
  }
  
  @Override
  protected Size adjustSize(Size size) {
    int width = size.getWidth();
    if (width != -1 && width > 50) {
      width -= getContainerTarget().getMargins(Side.LEFT, Side.RIGHT);
      size.setWidth(width);
    }
    return size;
  }

}
