/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.form;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.sencha.gxt.cell.core.client.form.TriggerFieldCell;
import com.sencha.gxt.cell.core.client.form.TriggerFieldCell.TriggerFieldAppearance;
import com.sencha.gxt.data.shared.Store;
import com.sencha.gxt.data.shared.Store.StoreFilter;
import com.sencha.gxt.widget.core.client.event.TriggerClickEvent;
import com.sencha.gxt.widget.core.client.event.TriggerClickEvent.TriggerClickHandler;

public abstract class StoreFilterField<T> extends TriggerField<T> {

  public interface StoreFilterFieldAppearance extends TriggerFieldAppearance {

  }
  static class StoreFilterFieldCell<T> extends TriggerFieldCell<T> {
    public StoreFilterFieldCell() {
      super(GWT.<StoreFilterFieldAppearance> create(StoreFilterFieldAppearance.class)) ;
    }
  }

  protected List<Store<T>> stores = new ArrayList<Store<T>>();
  
  protected StoreFilter<T> filter;

  public StoreFilterField() {
    super(new StoreFilterFieldCell<T>());
    setAutoValidate(true);
    setValidateOnBlur(false);

    filter = new StoreFilter<T>() {
      @Override
      public boolean select(Store<T> store, T parent, T item) {
        String v = getText();
        return doSelect(store, parent, item, v);
      }
    };

    redraw();
    
    addTriggerClickHandler(new TriggerClickHandler() {
      
      @Override
      public void onTriggerClick(TriggerClickEvent event) {
        StoreFilterField.this.onTriggerClick(event);
      }
    });
  }

  public void bind(Store<T> store) {
    stores.add(store);
  };

  public void unbind(Store<T> store) {
    stores.remove(store);
  }

  protected void applyFilters(Store<T> store) {
    if (getText().length() > 0) {
      store.addFilter(filter);
      store.setEnableFilters(true);
    } else {
      store.removeFilter(filter);
    }
  }

  protected abstract boolean doSelect(Store<T> store, T parent, T item, String filter);

  protected void onFilter() {
    for (Store<T> s : stores) {
      applyFilters(s);
    }
    focus();
  }
  
  protected void onTriggerClick(TriggerClickEvent event) {
    setValue(null);
    onFilter();
  }

  protected boolean validateValue(T value) {
    boolean ret = super.validateValue(value);
    onFilter();
    return ret;
  }

}
