/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.KeyCodes;
import com.google.gwt.user.client.Event;
import com.sencha.gxt.core.shared.event.GroupingHandlerRegistration;
import com.sencha.gxt.widget.core.client.event.CellMouseDownEvent;
import com.sencha.gxt.widget.core.client.event.CellMouseDownEvent.CellMouseDownHandler;
import com.sencha.gxt.widget.core.client.event.ViewReadyEvent;
import com.sencha.gxt.widget.core.client.event.ViewReadyEvent.ViewReadyHandler;
import com.sencha.gxt.widget.core.client.grid.Grid.GridCell;

public class CellSelectionModel<M> extends GridSelectionModel<M> {

  public class CellSelection {
    public int cell;
    public M model;
    public int row;

    public CellSelection(M model, int row, int cell) {
      this.model = model;
      this.row = row;
      this.cell = cell;
    }
  }
  
  private class Handler implements ViewReadyHandler, CellMouseDownHandler {

    @Override
    public void onCellMouseDown(CellMouseDownEvent event) {
      handleMouseDown(event);
    }

    @Override
    public void onViewReady(ViewReadyEvent event) {
      if (selection != null) {
        selectCell(selection.row, selection.cell);
      }
    }
  }

  protected CellSelection selection;

  private SelectionModelCallback callback = new SelectionModelCallback(this);
  private Handler handler = new Handler();
  
  @Override
  public void bindGrid(Grid<M> grid) {
    if (this.grid != null) {
      if (handlerRegistration != null) {
        handlerRegistration.removeHandler();
        handlerRegistration = null;
      }

      keyNav.bind(null);
      bind(null);
    }
    this.grid = grid;
    if (grid != null) {
      grid.setTrackMouseOver(false);
      handlerRegistration = new GroupingHandlerRegistration();
      
      handlerRegistration.add(grid.addViewReadyHandler(handler));
      handlerRegistration.add(grid.addCellMouseDownHandler(handler));
      keyNav.bind(grid);
      bind(grid.getStore());
    }
  }
  
  @Override
  public void deselectAll() {
    if (selection != null) {
      // index may change with tree grid on expand / collapse
      // ask store for current row index
      int row = listStore.indexOf(selection.model);
      if (grid.isViewReady()) {
        grid.getView().onCellDeselect(row, selection.cell);
      }
      selection = null;
    }
  }
  
  /**
   * Returns the selected cell.
   * 
   * @return the selection cell
   */
  public CellSelection getSelectCell() {
    return selection;
  }
  
  @Override
  public M getSelectedItem() {
    return selection != null ? selection.model : null;
  }
  
  @SuppressWarnings("unchecked")
  @Override
  public List<M> getSelectedItems() {
    return selection != null ? new ArrayList<M>(Arrays.asList(selection.model)) : new ArrayList<M>();
  }
  
  /**
   * Selects the cell.
   * 
   * @param row the row index
   * @param cell the cell index
   */
  public void selectCell(int row, int cell) {
    deselectAll();
    M m = listStore.get(row);
    selection = new CellSelection(m, row, cell);
    if (grid.isViewReady()) {
      grid.getView().onCellSelect(row, cell);
      grid.getView().focusCell(row, cell, true);
    }
  }
  
  @Override
  protected void onUpdate(M model) {
    if (selection != null && selection.model == model) {
      grid.getView().onCellSelect(selection.row, selection.cell);
      grid.getView().focusCell(selection.row, selection.cell, true);
    }
  };
  
  protected void handleMouseDown(CellMouseDownEvent event) {
    Event e = event.getEvent();
    if (e.getButton() != Event.BUTTON_LEFT || isLocked()) {
      return;
    }
    selectCell(event.getRowIndex(), event.getCellIndex());
  }
  
  @Override
  protected void onKeyDown(NativeEvent e) {

  }
  
  @Override
  protected void onKeyPress(NativeEvent e) {
    String tag = e.getEventTarget().<Element>cast().getTagName();
    String cls = e.getEventTarget().<Element>cast().getClassName();
    if (tag.equals("INPUT") && !cls.equals("_focus")) {
      return;
    }
    
    if (selection == null) {
      e.preventDefault();
      e.stopPropagation();
      GridCell cell = grid.walkCells(0, 0, 1, callback);
      if (cell != null) {
        selectCell(cell.getRow(), cell.getCol());
      }
      return;
    }
    
    int r = selection.row;
    int c = selection.cell;

    GridCell newCell = null;
    
    switch (e.getKeyCode()) {
      case KeyCodes.KEY_HOME:
        if (enableNavKeys) {
          newCell = grid.walkCells(selection.row, 0, 1, callback);
        }
        break;
      case KeyCodes.KEY_END:
        if (enableNavKeys) {
          newCell = grid.walkCells(selection.row, grid.getColumnModel().getColumnCount() - 1, -1, callback);
        }
        break;
      case KeyCodes.KEY_PAGEUP:
        if (enableNavKeys) {
          newCell = grid.walkCells(0, 0, 1, callback);
        }
        break;
      case KeyCodes.KEY_PAGEDOWN:
        if (enableNavKeys) {
          int idx = listStore.indexOf(listStore.get(listStore.size() - 1));
          newCell = grid.walkCells(idx, 0, 1, callback);
        }
        break;
      case KeyCodes.KEY_TAB:
        if (e.getShiftKey()) {
          newCell = grid.walkCells(r, c - 1, -1, callback);
        } else {
          newCell = grid.walkCells(r, c + 1, 1, callback);
        }
        break;
      case KeyCodes.KEY_DOWN: {
        newCell = grid.walkCells(r + 1, c, 1, callback);
        break;
      }
      case KeyCodes.KEY_UP: {
        newCell = grid.walkCells(r - 1, c, -1, callback);
        break;
      }
      case KeyCodes.KEY_LEFT:
        newCell = grid.walkCells(r, c - 1, -1, callback);
        break;
      case KeyCodes.KEY_RIGHT:
        newCell = grid.walkCells(r, c + 1, 1, callback);
        break;
      case KeyCodes.KEY_ENTER:
//        if (grid.editSupport != null) {
//          if (!grid.editSupport.isEditing()) {
//            grid.editSupport.startEditing(r, c);
//            e.stopEvent();
//            return;
//          }
//        }
        break;
    }
    
    if (newCell != null) {
      selectCell(newCell.getRow(), newCell.getCol());
      e.preventDefault();
      e.stopPropagation();
    }
  }

  @Override
  protected void onKeyUp(NativeEvent e) {

  }
}
