/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid;

import java.util.List;

import com.google.gwt.cell.client.AbstractCell;
import com.google.gwt.dom.client.Element;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.user.client.Event;
import com.sencha.gxt.core.client.IdentityValueProvider;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.data.shared.event.StoreClearEvent;
import com.sencha.gxt.widget.core.client.event.HeaderClickEvent;
import com.sencha.gxt.widget.core.client.event.HeaderClickEvent.HeaderClickHandler;
import com.sencha.gxt.widget.core.client.event.RowClickEvent;
import com.sencha.gxt.widget.core.client.event.RowMouseDownEvent;

/**
 * A grid selection model. To use, add the column config to the column model
 * using {@link #getColumn()}.
 * 
 * <p>
 * This selection mode defaults to SelectionMode.MULTI and also supports
 * SelectionMode.SIMPLE. With SIMPLE, the control and shift keys do not need to
 * be pressed for multiple selections.
 * 
 * @param <M> the model data type
 */
public class CheckBoxSelectionModel<M> extends GridSelectionModel<M> {

  protected ColumnConfig<M, M> config;

  public CheckBoxSelectionModel(IdentityValueProvider<M> valueProvider) {
    config = newColumnConfig(valueProvider);
    config.setColumnClassName("checker");
    config.setWidth(20);
    config.setSortable(false);
    config.setResizable(false);
    config.setFixed(true);
    config.setMenuDisabled(true);

    config.setCell(new AbstractCell<M>() {
      @Override
      public void render(Context context, M value, SafeHtmlBuilder sb) {
        sb.appendHtmlConstant("<div class='x-grid3-row-checker'>&#160;</div>");
      }
    });
  }

  /**
   * Returns the column config.
   * 
   * @return the column config
   */
  public ColumnConfig<M, M> getColumn() {
    return config;
  }

  @Override
  public void bindGrid(Grid<M> grid) {
    super.bindGrid(grid);

    grid.addHeaderClickHandler(new HeaderClickHandler() {

      @Override
      public void onHeaderClick(HeaderClickEvent event) {
        handleHeaderClick(event);
      }
    });
  }

  protected void handleHeaderClick(HeaderClickEvent event) {
    ColumnConfig<M, ?> c = grid.getColumnModel().getColumn(event.getColumnIndex());
    if (c == config) {
      XElement hd = event.getEvent().getEventTarget().<Element> cast().getParentElement().cast();
      boolean isChecked = hd.hasClassName("x-grid3-hd-checker-on");
      hd.getFirstChildElement().setAttribute("aria-selected", isChecked ? "false" : "true");
      if (isChecked) {
        setChecked(false);
        deselectAll();
      } else {
        setChecked(true);
        selectAll();
      }
    }
  }

  @Override
  protected void handleRowClick(RowClickEvent event) {
    Element target = event.getEvent().getEventTarget().cast();
    if (target.getClassName().equals("x-grid3-row-checker")) {
      return;
    }
    super.handleRowClick(event);
  }

  @Override
  protected void handleRowMouseDown(RowMouseDownEvent event) {
    boolean left = event.getEvent().getButton() == Event.BUTTON_LEFT;
    Element target = event.getEvent().getEventTarget().cast();

    if (left && target.getClassName().equals("x-grid3-row-checker")) {
      M model = listStore.get(event.getRowIndex());
      if (model != null) {
        if (isSelected(model)) {
          deselect(model);
        } else {
          select(model, true);
        }
      }
    } else {
      super.handleRowMouseDown(event);
    }
  }

  protected ColumnConfig<M, M> newColumnConfig(IdentityValueProvider<M> valueProvider) {
    return new ColumnConfig<M, M>(valueProvider);
  }

  @Override
  protected void onAdd(List<? extends M> models) {
    super.onAdd(models);
    updateHeaderCheckBox();
  };

  @Override
  protected void onClear(StoreClearEvent<M> event) {
    super.onClear(event);
    updateHeaderCheckBox();
  }

  protected void onRemove(M model) {
    super.onRemove(model);
    updateHeaderCheckBox();
  };

  @Override
  protected void onSelectChange(M model, boolean select) {
    super.onSelectChange(model, select);
    updateHeaderCheckBox();
  }

  protected void setChecked(boolean checked) {
    if (grid.isViewReady()) {
      XElement hd = grid.getView().innerHd.child("div.x-grid3-hd-checker");
      if (hd != null) {
        hd.getParentElement().<XElement> cast().setClassName("x-grid3-hd-checker-on", checked);
      }
    }
  }

  private void updateHeaderCheckBox() {
    setChecked(getSelection().size() == listStore.size());
  }

}
