/**
 * Ext GWT 3.0.0-beta2 - Ext for GWT
 * Copyright(c) 2007-2011, Sencha, Inc.
 * licensing@sencha.com
 *
 * http://sencha.com/license
 */
package com.sencha.gxt.widget.core.client.grid;

import java.util.ArrayList;
import java.util.List;

import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.JsArray;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NodeList;
import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.event.shared.HandlerManager;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.user.client.Event;
import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.core.client.XTemplates;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.util.Point;
import com.sencha.gxt.data.shared.SortDir;
import com.sencha.gxt.data.shared.SortInfo;
import com.sencha.gxt.data.shared.SortInfoBean;
import com.sencha.gxt.data.shared.Store.StoreSortInfo;
import com.sencha.gxt.messages.client.DefaultMessages;
import com.sencha.gxt.widget.core.client.event.CollapseItemEvent;
import com.sencha.gxt.widget.core.client.event.CollapseItemEvent.CollapseItemHandler;
import com.sencha.gxt.widget.core.client.event.CollapseItemEvent.HasCollapseItemHandlers;
import com.sencha.gxt.widget.core.client.event.ExpandItemEvent;
import com.sencha.gxt.widget.core.client.event.ExpandItemEvent.ExpandItemHandler;
import com.sencha.gxt.widget.core.client.event.ExpandItemEvent.HasExpandItemHandlers;
import com.sencha.gxt.widget.core.client.menu.CheckMenuItem;
import com.sencha.gxt.widget.core.client.menu.Item;
import com.sencha.gxt.widget.core.client.menu.Menu;
import com.sencha.gxt.widget.core.client.menu.MenuItem;
import com.sencha.gxt.widget.core.client.menu.SeparatorMenuItem;

/**
 * GridView that groups data based on a given grouping column.
 * 
 * @param <M> the model type
 */
public class GroupingView<M> extends GridView<M> implements HasCollapseItemHandlers<List<M>>,
    HasExpandItemHandlers<List<M>> {

  @SuppressWarnings("rawtypes")
  public interface DefaultGroupHeaderTemplate extends XTemplates, GroupHeaderTemplate {
    @Override
    @XTemplate("{groupInfo.value}")
    public SafeHtml renderGroupHeader(GroupingData groupInfo);
  }

  @SuppressWarnings("rawtypes")
  public interface DefaultGroupingViewTemplate extends XTemplates, GroupingViewTemplate {
    @Override
    @XTemplate(source = "GroupingView.html")
    SafeHtml renderGroup(SafeHtml groupHeader, SafeHtml renderedRows, SafeHtml groupSummary);
  }

  public interface GroupHeaderTemplate<M> {
    SafeHtml renderGroupHeader(GroupingData<M> groupInfo);
  }

  public static class GroupingData<M> {
    private final Object value;
    private final int startRow;
    private final List<M> items = new ArrayList<M>();
    private boolean collapsed;

    public GroupingData(Object value, int startRow) {
      assert value != null;
      this.value = value;
      this.startRow = startRow;
    }

    @Override
    public boolean equals(Object other) {
      if (other instanceof GroupingData) {
        return ((GroupingData<?>) other).value.equals(value);
      }
      return false;
    }

    public List<M> getItems() {
      return items;
    }

    public int getStartRow() {
      return startRow;
    }

    public Object getValue() {
      return value;
    }

    @Override
    public int hashCode() {
      return value.hashCode();
    }

    public boolean isCollapsed() {
      return collapsed;
    }

    public void setCollapsed(boolean collapsed) {
      this.collapsed = collapsed;
    }

  }

  public interface GroupingViewTemplate<M> {
    SafeHtml renderGroup(SafeHtml groupHeader, SafeHtml rows, SafeHtml groupSummary);
  }

  public interface GroupSummaryTemplate<M> {
    SafeHtml renderGroupSummary(GroupingData<M> groupInfo);
  }

  protected ColumnConfig<M, ?> lastGroupField;
  protected ColumnConfig<M, ?> groupingColumn;

  private GroupingViewTemplate<M> groupViewTemplate = GWT.create(DefaultGroupingViewTemplate.class);
  private GroupHeaderTemplate<M> groupTemplate = GWT.create(DefaultGroupHeaderTemplate.class);
  private GroupSummaryTemplate<M> groupSummaryTemplate;

  private boolean enableGroupingMenu = true;
  private boolean enableNoGroups = true;
  private boolean showGroupedColumn = true;
  private boolean startCollapsed;
  protected boolean enableGrouping;
  private boolean isUpdating = false;
  private StoreSortInfo<M> lastStoreSort;
  private SortInfo lastSort;
  private HandlerManager handlerManager = new HandlerManager(this);

  @Override
  public HandlerRegistration addCollapseHandler(CollapseItemHandler<List<M>> handler) {
    return handlerManager.addHandler(CollapseItemEvent.getType(), handler);
  }

  @Override
  public HandlerRegistration addExpandHandler(ExpandItemHandler<List<M>> handler) {
    return handlerManager.addHandler(ExpandItemEvent.getType(), handler);
  }

  /**
   * Collapses all groups.
   */
  public void collapseAllGroups() {
    toggleAllGroups(false);
  }

  /**
   * Expands all groups.
   */
  public void expandAllGroups() {
    toggleAllGroups(true);
  }

  public GroupSummaryTemplate<M> getGroupSummaryTemplate() {
    return groupSummaryTemplate;
  }

  public GroupHeaderTemplate<M> getGroupTemplate() {
    return groupTemplate;
  }

  public GroupingViewTemplate<M> getGroupViewTemplate() {
    return groupViewTemplate;
  }

  @SuppressWarnings({"unchecked", "rawtypes"})
  public void groupBy(ColumnConfig<M, ?> column) {
    if (column != groupingColumn) {
      // remove the existing group, if any
      if (groupingColumn != null) {
        if (grid.getLoader() == null || !grid.getLoader().isRemoteSort()) {
          assert lastStoreSort != null && ds.getSortInfo().contains(lastStoreSort);
          ds.getSortInfo().remove(lastStoreSort);
        } else {
          assert lastSort != null;
          grid.getLoader().removeSortInfo(lastSort);
        }
      } else {
        assert lastStoreSort == null;
      }

      // set the new one
      groupingColumn = column;
      if (column != null) {
        if (grid.getLoader() == null || !grid.getLoader().isRemoteSort()) {
          lastStoreSort = new StoreSortInfo<M>((ValueProvider) column.getValueProvider(), SortDir.ASC);
          ds.addSortInfo(0, lastStoreSort);// this triggers the sort
        } else {
          lastSort = new SortInfoBean(column.getValueProvider(), SortDir.ASC);
          grid.getLoader().addSortInfo(0, lastSort);
          grid.getLoader().load();
        }
      } else {// new column == null
        lastStoreSort = null;
        lastSort = null;
        // full redraw without groups
        refresh(false);
      }
    }
  }

  public boolean isEnabledNoGroups() {
    return enableNoGroups;
  }

  public boolean isEnableGroupingMenu() {
    return enableGroupingMenu;
  }

  public boolean isShowGroupedColumn() {
    return showGroupedColumn;
  }

  public boolean isStartCollapsed() {
    return startCollapsed;
  }

  public void setEnableGroupingMenu(boolean enableGroupingMenu) {
    this.enableGroupingMenu = enableGroupingMenu;
  }

  public void setEnableNoGroups(boolean enableNoGroups) {
    this.enableNoGroups = enableNoGroups;
  }

  public void setGroupSummaryTemplate(GroupSummaryTemplate<M> groupSummaryTemplate) {
    this.groupSummaryTemplate = groupSummaryTemplate;
  }

  public void setGroupTemplate(GroupHeaderTemplate<M> groupTemplate) {
    this.groupTemplate = groupTemplate;
  }

  public void setGroupViewTemplate(GroupingViewTemplate<M> groupTemplate) {
    this.groupViewTemplate = groupTemplate;
  }

  public void setShowGroupedColumn(boolean showGroupedColumn) {
    this.showGroupedColumn = showGroupedColumn;
  }

  public void setStartCollapsed(boolean startCollapsed) {
    this.startCollapsed = startCollapsed;
  }

  public void toggleAllGroups(boolean expanded) {
    NodeList<Element> groups = getGroups();
    for (int i = 0; i < groups.getLength(); i++) {
      toggleGroup(groups.getItem(i), expanded);
    }
  }

  protected Menu createContextMenu(final int colIndex) {
    Menu menu = super.createContextMenu(colIndex);

    if (menu != null && enableGroupingMenu) {
      if (cm.isGroupable(colIndex)) {
        MenuItem groupBy = new MenuItem(DefaultMessages.getMessages().groupingView_groupByText());
        // groupBy.setIcon(icon);//TODO
        groupBy.addSelectionHandler(new SelectionHandler<Item>() {
          @Override
          public void onSelection(SelectionEvent<Item> event) {
            groupBy(cm.getColumn(colIndex));
          }
        });
        menu.add(new SeparatorMenuItem());
        menu.add(groupBy);
      }
      if (enableGrouping && enableNoGroups) {
        final CheckMenuItem showInGroups = new CheckMenuItem(DefaultMessages.getMessages().groupingView_showGroupsText());
        showInGroups.setChecked(true);
        showInGroups.addSelectionHandler(new SelectionHandler<Item>() {
          @Override
          public void onSelection(SelectionEvent<Item> event) {
            if (showInGroups.isChecked()) {
              groupBy(cm.getColumn(colIndex));
            } else {
              groupBy(null);
            }
          }
        });
        menu.add(showInGroups);
      }
    }

    return menu;
  }

  @Override
  protected SafeHtml doRender(List<ColumnData> cs, List<M> rows, int startRow) {

    // int colIndex = cm.indexOf(groupingColumn);

    enableGrouping = groupingColumn != null;

    if (!enableGrouping || isUpdating) {
      return super.doRender(cs, rows, startRow);
    }

    // String gstyle = "width:" + getTotalWidth() + "px;";
    // String gidPrefix = grid.getId();

    // ColumnConfig<M, ?> cfg = cm.getColumn(colIndex);

    // String prefix = showGroupName ? cfg.getHeader() + ": " : "";

    GroupingData<M> curGroup = null;
    // String gid = null;

    List<GroupingData<M>> groups = new ArrayList<GroupingData<M>>();

    // iterate through each item, creating a new group as needed. Assumes the
    // list is sorted
    for (int j = 0; j < rows.size(); j++) {
      M model = rows.get(j);

      int rowIndex = (j + startRow);

      // the value for the group field
      Object gvalue = groupingColumn.getValueProvider().getValue(model);

      // the rendered group value
      // String g = getGroup(gvalue, model, rowIndex, colIndex, ds);

      if (curGroup == null || !curGroup.getValue().equals(gvalue)) {
        // gid = getGroupId(gidPrefix, groupingColumn, g);

        // TODO state
        // boolean isCollapsed = state.get(gid) != null ? !state.get(gid) :
        // startCollapsed;
        // String gcls = startCollapsed ? "x-grid-group-collapsed" : "";

        curGroup = new GroupingData<M>(gvalue, rowIndex);
        // curGroup.group = g;
        // curGroup.text = prefix + g;
        // curGroup.groupId = gid;
        // curGroup.style = gstyle;
        // curGroup.css = gcls;
        curGroup.setCollapsed(true);
        curGroup.getItems().add(model);
        assert !groups.contains(curGroup);
        groups.add(curGroup);

      } else {
        curGroup.getItems().add(model);
      }
      // model.set("_groupId", gid);

    }

    // for (GroupingData<M> group : groups) {
    //
    // }

    // TODO support a grouping renderer, perhaps in the
    // for (GroupingData<M> group : groups) {
    // if (groupRenderer != null) {
    // String g = groupRenderer.render(group);
    // if (g == null || g.equals("")) {
    // g = "&nbsp;";
    // }
    // group.group = g;
    // }
    // }

    SafeHtmlBuilder buf = new SafeHtmlBuilder();

    for (int i = 0, len = groups.size(); i < len; i++) {
      GroupingData<M> g = groups.get(i);
      SafeHtml renderedRows = super.doRender(cs, g.getItems(), g.getStartRow());
      renderGroup(buf, g, renderedRows);
    }

    return buf.toSafeHtml();
  }

  @Override
  protected <N> void doSort(int colIndex, SortDir sortDir) {
    ColumnConfig<M, ?> column = cm.getColumn(colIndex);
    if (groupingColumn != null) {
      if (grid.getLoader() == null || !grid.getLoader().isRemoteSort()) {
        // first sort is lastStoreSort
        assert lastStoreSort != null;
        ds.getSortInfo().clear();

        // These casts can fail, but in dev mode the exception will be caught by
        // the
        // try/catch, unless there are no items in the Store
        @SuppressWarnings({"unchecked", "rawtypes"})
        ValueProvider<M, Comparable> vp = (ValueProvider) column.getValueProvider();
        @SuppressWarnings("unchecked")
        StoreSortInfo<M> sortInfo = new StoreSortInfo<M>(vp, sortDir);

        if (sortDir == null && storeSortInfo != null && storeSortInfo.getValueProvider() == vp) {
          sortInfo.setDirection(storeSortInfo.getDirection() == SortDir.ASC ? SortDir.DESC : SortDir.ASC);
        } else if (sortDir == null) {
          sortInfo.setDirection(SortDir.ASC);
        }

        ds.getSortInfo().add(0, lastStoreSort);
        ds.getSortInfo().add(1, sortInfo);

        if (GWT.isProdMode()) {
          ds.applySort(false);
        } else {
          try {
            // applySort will apply its sort when called, which might trigger an
            // exception if the column passed in's data isn't Comparable
            ds.applySort(false);
          } catch (ClassCastException ex) {
            GWT.log("Column can't be sorted " + column.getValueProvider().getPath() + " is not Comparable. ", ex);
            throw ex;
          }
        }
      } else {
        assert lastSort != null;
        ValueProvider<M, ?> vp = column.getValueProvider();
        grid.getLoader().clearSortInfo();
        grid.getLoader().addSortInfo(0, lastSort);
        grid.getLoader().addSortInfo(1, new SortInfoBean(vp, sortDir));
        grid.getLoader().load();
      }

    } else {
      super.doSort(colIndex, sortDir);
    }
  }

  protected List<GroupingData<M>> getGroupData() {
    List<GroupingData<M>> groups = new ArrayList<GroupingData<M>>();

    GroupingData<M> curGroup = null;
    for (int i = 0, len = ds.size(); i < len; i++) {
      M m = ds.get(i);
      Object gvalue = groupingColumn.getValueProvider().getValue(m);
      if (curGroup == null || !curGroup.getValue().equals(gvalue)) {
        curGroup = new GroupingData<M>(gvalue, i);
        curGroup.setCollapsed(true);
        curGroup.getItems().add(m);
        assert !groups.contains(curGroup);
        groups.add(curGroup);
      } else {
        curGroup.getItems().add(m);
      }
    }

    return groups;
  }

  protected NodeList<Element> getGroups() {
    if (!enableGrouping) {
      return JsArray.createArray().cast();
    }
    return mainBody.getChildNodes().cast();
  }

  @Override
  protected NodeList<com.google.gwt.dom.client.Element> getRows() {
    if (!enableGrouping || !hasRows()) {
      return super.getRows();
    }
    NodeList<Element> gs = getGroups();
    JsArray<Element> rows = JsArray.createArray().cast();
    for (int i = 0, len = gs.getLength(); i < len; i++) {
      NodeList<Element> g = gs.getItem(i).getChildNodes().getItem(1).getChildNodes().cast();
      for (int j = 0, len2 = g.getLength(); j < len2; j++) {
        rows.push(g.getItem(j));
      }
    }
    return rows.cast();
  }

  @Override
  protected StoreSortInfo<M> getSortState() {
    if (ds.getSortInfo().size() > 1) {
      return ds.getSortInfo().get(1);
    }
    return null;
  }

  protected boolean isExpanded(Element group) {
    return false;
  }

  @Override
  protected void onAdd(List<M> models, int index) {
    if (enableGrouping) {
      Point ss = getScrollState();
      refresh(false);
      restoreScroll(ss);
    } else {
      super.onAdd(models, index);
    }
  }

  @Override
  protected void onMouseDown(Event ge) {
    super.onMouseDown(ge);
    XElement head = ge.getEventTarget().cast();
    // TODO appearance
    head = head.findParent(".x-grid-group-hd", 10);
    if (head != null) {
      ge.stopPropagation();
      // TODO appearance
      XElement parent = head.getParentElement().<XElement> cast();
      toggleGroup(parent, parent.hasClassName("x-grid-group-collapsed"));
    }
  }

  @Override
  protected void onRemove(M m, int index, boolean isUpdate) {
    Element parentToRemove = null;
    if (enableGrouping) {
      Element r = getRow(index).cast();

      // TODO appearance this
      Element groupContainer = r.getParentElement().cast();
      if (groupContainer.getChildCount() == 1) {
        parentToRemove = groupContainer.getParentElement().cast();
      }
    }
    super.onRemove(m, index, isUpdate);
    if (parentToRemove != null) {
      parentToRemove.removeFromParent();
    }
  }

  @Override
  protected void refreshRow(int row) {
    isUpdating = true;
    super.refreshRow(row);
    isUpdating = false;
  }

  protected void renderGroup(SafeHtmlBuilder buf, GroupingData<M> g, SafeHtml renderedRows) {
    SafeHtml summary = renderGroupSummary(g);
    buf.append(groupViewTemplate.renderGroup(renderGroupHeader(g), renderedRows, summary));
  }

  protected SafeHtml renderGroupHeader(GroupingData<M> groupInfo) {
    return groupTemplate.renderGroupHeader(groupInfo);
  }

  protected SafeHtml renderGroupSummary(GroupingData<M> groupInfo) {
    if (groupSummaryTemplate != null) {
      return groupSummaryTemplate.renderGroupSummary(groupInfo);
    } else {
      return null;
    }
  }

  @Override
  protected SafeHtml renderRows(int startRow, int endRow) {
    boolean eg = groupingColumn != null;
    if (!showGroupedColumn) {
      int colIndex = cm.indexOf(groupingColumn);
      if (!eg && lastGroupField != null) {
        mainBody.setInnerHTML("");
        cm.setHidden(cm.indexOf(lastGroupField), false);
        lastGroupField = groupingColumn;
      } else if (eg && (lastGroupField == null || lastGroupField == groupingColumn)) {
        lastGroupField = groupingColumn;
        cm.setHidden(colIndex, true);
      } else if (eg && lastGroupField != null && !groupingColumn.equals(lastGroupField)) {
        mainBody.setInnerHTML("");
        int oldIndex = cm.indexOf(lastGroupField);
        cm.setHidden(oldIndex, false);
        lastGroupField = groupingColumn;
        cm.setHidden(colIndex, true);
      } else {
        assert false : "No update occuring - acceptable?";
      }
    }
    return super.renderRows(startRow, endRow);
  }

  protected void toggleGroup(Element g, boolean expanded) {
    assert g != null;
    // TODO editing
    // if (grid.editSupport != null) {
    // grid.editSupport.stopEditing();
    // }
    g.<XElement> cast().setClassName("x-grid-group-collapsed", !expanded);
    // g.getFirstChildElement().setAttribute("aria-expanded", expanded ? "true"
    // : "false");
    calculateVBar(false);
    if (expanded) {
      handlerManager.fireEvent(new ExpandItemEvent<List<M>>(null));
    } else {
      handlerManager.fireEvent(new CollapseItemEvent<List<M>>(null));
    }
  }
}
